from marshmallow import EXCLUDE, Schema

from ..fields.objectid import ID


class BaseSchema(Schema):
    id = ID(description='ID', dump_only=True)

    class Meta:
        strict = True
        ordered = True
        unknown = EXCLUDE



import sys
from importlib import import_module

import freenit.schemas.user
from flask import Flask, send_file
from flask_collect import Collect
from flask_cors import CORS
from flask_jwt_extended import JWTManager
from flask_security import Security
from freenit.schemas.paging import PageOutSchema

from . import cli
from .api import create_api
from .utils import sendmail


def sqlinit(app):
    from flask_security import PeeweeUserDatastore
    from .db import db
    db.init_app(app)
    app.db = db
    try:
        User = import_module(f"{app.config['NAME']}.models.sql.user").User
    except ModuleNotFoundError:
        User = import_module(f"{app.config['NAME']}.models.user").User
    try:
        role_module = import_module(f"{app.config['NAME']}.models.role")
    except ModuleNotFoundError:
        role_module = import_module(f"{app.config['NAME']}.models.sql.role")
    Role = role_module.Role
    UserRoles = role_module.UserRoles
    app.user_datastore = PeeweeUserDatastore(
        app.db,
        User,
        Role,
        UserRoles,
    )


def mongoinit(app):
    from flask_security import MongoEngineUserDatastore
    from flask_mongoengine import MongoEngine
    app.db = MongoEngine(app)
    try:
        User = import_module(f"{app.config['NAME']}.models.user").User
    except ModuleNotFoundError:
        User = import_module(f"{app.config['NAME']}.models.mongo.user").User
    try:
        Role = import_module(f"{app.config['NAME']}.models.role").Role
    except ModuleNotFoundError:
        Role = import_module(f"{app.config['NAME']}.models.mongo.role").Role
    app.user_datastore = MongoEngineUserDatastore(app.db, User, Role)


def create_app(
    config,
    app=None,
    schemas={},
    dbtype='sql',
    name=__name__,
    **kwargs,
):
    if app is None:
        app = Flask(name, **kwargs)
        app.config.from_object(config)

    @app.route('/media/<path:path>')
    def send_media(path):
        fullPath = f"{app.config['MEDIA_PATH']}/{path}"
        if fullPath[0] != '/':
            fullPath = f"/{app.config['PROJECT_ROOT']}/{fullPath}"
        try:
            return send_file(fullPath)
        except FileNotFoundError:
            return 'No such file', 404

    app.sendmail = lambda message: sendmail(app.config, message)
    app.collect = Collect(app)
    app.dbtype = dbtype
    if dbtype in ['sql', 'all']:
        sqlinit(app)
    elif dbtype == 'mongo':
        mongoinit(app)
    app.security = Security(app, app.user_datastore)
    user_module = schemas.get('user', None)
    if user_module is None:

        class UserSchema(freenit.schemas.user.BaseUserSchema):
            pass
    else:
        UserSchema = import_module(user_module).UserSchema
    setattr(freenit.schemas.user, 'UserSchema', UserSchema)
    PageOutSchema(UserSchema, sys.modules['freenit.schemas.user'])
    role_module = schemas.get('role', None)
    if role_module is None:

        class RoleSchema(freenit.schemas.role.BaseRoleSchema):
            pass
    else:
        RoleSchema = import_module(role_module).RoleSchema
    setattr(freenit.schemas.role, 'RoleSchema', RoleSchema)
    PageOutSchema(RoleSchema, sys.modules['freenit.schemas.role'])
    app.jwt = JWTManager(app)
    create_api(app)
    app.cors = CORS(app, supports_credentials=True)
    cli.register(app)

    return app



from .base import Base


class TestUser(Base):
    def test_get_all(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.get('users.list')
        assert response.status_code == 200

    def test_post(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        data = {
            'email': 'some@example.com',
            'password': 'password',
        }
        response = self.post('users.list', data, 'access')
        assert response.status_code == 200
        json_data = response.get_json()
        assert data['email'] == json_data['email']
        newuser = user.get(id=json_data['id'])
        assert newuser.email == data['email']

    def test_get(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.get('users.detail', user_id=user.id)
        assert response.status_code == 200

    def test_patch(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        data = {
            'email': 'other@example.com',
        }
        response = self.patch('users.detail', data, 'access', user_id=user.id)
        assert response.status_code == 200
        user = user.get(id=user.id)
        assert user.email == data['email']

    def test_delete(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        newuser = user_factory()
        newuser.save()
        response = self.delete('users.detail', 'access', user_id=newuser.id)
        assert response.status_code == 200



from .base import Base


class TestAuth(Base):
    def test_me(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.get('profile.profile')
        assert response.status_code == 200

    def test_logout(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.post('auth.logout', {})
        assert response.status_code == 200

    def test_refresh(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.post('auth.refresh', {}, 'refresh')
        assert response.status_code == 200



try:
    from local_config import DevConfig
except ImportError:
    from common_config import DevConfig

try:
    from local_config import TestConfig
except ImportError:
    from common_config import TestConfig

try:
    from local_config import ProdConfig
except ImportError:
    from common_config import ProdConfig

configs = {
    'development': DevConfig,
    'testing': TestConfig,
    'production': ProdConfig,
    'default': ProdConfig,
}



import os

import pytest
from peewee_migrate import Router
from peewee_migrate.router import DEFAULT_MIGRATE_DIR
from pytest_factoryboy import register

from config import configs
from freenit import create_app

from .factories import AdminFactory, RoleFactory, UserFactory

register(UserFactory)
register(AdminFactory)
register(RoleFactory)


@pytest.fixture
def app():
    dbtype = os.environ.get('DBTYPE', 'sql')
    config = configs['testing']
    flask_app = create_app(config, dbtype=dbtype)
    if dbtype == 'sql':
        router = Router(
            flask_app.db.database,
            migrate_dir=f'{DEFAULT_MIGRATE_DIR}/main',
        )
        router.run()
    yield flask_app
    if dbtype == 'sql':
        flask_app.db.close_db('')
        current_path = os.path.dirname(__file__)
        os.remove('{}/../test.db'.format(current_path))



from math import ceil

from flask_smorest import abort
from marshmallow import fields

from .base import BaseSchema


def paginate(query, pagination):
    page = pagination.get('Page', 0)
    per_page = pagination.get('PerPage', 10)
    offset = page * per_page
    total = query.count()
    if offset > total:
        abort(409, message='Requested range out of boundaries')
    totalPages = ceil(total / float(per_page))
    data = []
    if getattr(query, '__sql__', None):
        data = query.limit(per_page).offset(offset)
    else:
        end = offset + per_page
        data = query[offset:end]
    return {
        'data': data,
        'pages': totalPages,
        'total': total,
    }


class PageInSchema(BaseSchema):
    Page = fields.Int()
    PerPage = fields.Int()


def PageOutSchema(schema, module):
    name = schema.__name__
    if name.endswith("Schema"):
        name = name[:-6] or name
    name += 'PageOutSchema'

    base = (schema, )
    PS = type(
        name,
        base,
        {
            'pages': fields.Integer(),
            'total': fields.Integer(),
            'data': fields.List(fields.Nested(schema)),
        }
    )
    setattr(module, name, PS)
    return PS



from flask_security import RoleMixin

from flask_mongoengine import Document
from mongoengine.fields import StringField


class Role(Document, RoleMixin):
    name = StringField(max_length=255)
    description = StringField(max_length=255)

    def __repr__(self):
        return '<Role %r>' % self.name



from flask import current_app
from flask_smorest import Blueprint, abort

from ..schemas.paging import PageInSchema, paginate
from ..schemas.role import RolePageOutSchema, RoleSchema, UserAssignSchema
from .methodviews import ProtectedMethodView

blueprint = Blueprint('roles', 'role')


@blueprint.route('', endpoint='list')
class RoleListAPI(ProtectedMethodView):
    @blueprint.arguments(PageInSchema(), location='headers')
    @blueprint.response(RolePageOutSchema)
    def get(self, pagination):
        """List roles"""
        Role = current_app.user_datastore.role_model
        if current_app.dbtype == 'sql':
            return paginate(Role.select(), pagination)
        else:
            return paginate(Role.objects.all(), pagination)

    @blueprint.arguments(RoleSchema)
    @blueprint.response(RoleSchema)
    def post(self, args):
        """Create role"""
        Role = current_app.user_datastore.role_model
        role = Role(**args)
        role.save()
        return role


@blueprint.route('/<role_id>', endpoint='detail')
class RoleAPI(ProtectedMethodView):
    @blueprint.response(RoleSchema)
    def get(self, role_id):
        """Get role details"""
        Role = current_app.user_datastore.role_model
        try:
            if current_app.dbtype == 'sql':
                role = Role.get(id=role_id)
            else:
                role = Role.objects.get(id=role_id)
        except Role.DoesNotExist:
            abort(404, message='Role not found')
        role.users = [user.user for user in role.users]
        return role

    @blueprint.arguments(RoleSchema(partial=True))
    @blueprint.response(RoleSchema)
    def patch(self, args, role_id):
        """Edit role"""
        Role = current_app.user_datastore.role_model
        try:
            if current_app.dbtype == 'sql':
                role = Role.get(id=role_id)
            else:
                role = Role.objects.get(id=role_id)
        except Role.DoesNotExist:
            abort(404, message='Role not found')
        for field in args:
            setattr(role, field, args[field])
        role.save()
        return role

    @blueprint.response(RoleSchema)
    def delete(self, role_id):
        """Remove role"""
        Role = current_app.user_datastore.role_model
        try:
            if current_app.dbtype == 'sql':
                role = Role.get(id=role_id)
            else:
                role = Role.objects.get(id=role_id)
        except Role.DoesNotExist:
            abort(404, message='Role not found')
        role.delete_instance()
        return role


@blueprint.route('/<role_id>/user', endpoint='user_assign')
class RoleUserAssignAPI(ProtectedMethodView):
    @blueprint.arguments(UserAssignSchema)
    @blueprint.response(RoleSchema)
    def post(self, args, role_id):
        """Assign user to role"""
        Role = current_app.user_datastore.role_model
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                role = Role.get(id=role_id)
            else:
                role = Role.objects.get(id=role_id)
        except Role.DoesNotExist:
            abort(404, message='No such role')
        try:
            user = User.get(id=args['id'])
        except User.DoesNotExist:
            abort(404, message='No such user')
        current_app.user_datastore.add_role_to_user(user, role)
        return role


@blueprint.route('/<role_id>/user/<user_id>', endpoint='user_deassign')
class RoleUserDeassignAPI(ProtectedMethodView):
    @blueprint.response(RoleSchema)
    def delete(self, role_id, user_id):
        """Remove user from role"""
        Role = current_app.user_datastore.role_model
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                role = Role.get(id=role_id)
            else:
                role = Role.objects.get(id=role_id)
        except Role.DoesNotExist:
            abort(404, message='No such role')
        try:
            if current_app.dbtype == 'sql':
                user = User.get(id=user_id)
            else:
                user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            abort(404, message='No such user')
        current_app.user_datastore.remove_role_from_user(user, role)
        return role



from flask_security import UserMixin

from flask_mongoengine import Document
from mongoengine.fields import (
    BooleanField,
    EmailField,
    ListField,
    ReferenceField,
    StringField
)

from .role import Role


class User(Document, UserMixin):
    active = BooleanField(default=False)
    admin = BooleanField()
    email = EmailField(unique=True)
    first_name = StringField(max_length=255)
    last_name = StringField(max_length=255)
    password = StringField(max_length=255)
    roles = ListField(ReferenceField(Role), default=[])

    def __repr__(self):
        return '<User %r>' % self.email



from flask_security import RoleMixin
from freenit.db import db
from peewee import CharField, ForeignKeyField, TextField

from .user import User

Model = db.Model


class Role(Model, RoleMixin):
    description = TextField(null=True)
    name = CharField(unique=True)


class UserRoles(Model):
    description = property(lambda self: self.role.description)
    name = property(lambda self: self.role.name)
    role = ForeignKeyField(Role, backref='users')
    user = ForeignKeyField(User, backref='roles')



from flask import current_app
from flask_jwt_extended import get_jwt_identity
from flask_security.utils import hash_password
from flask_smorest import Blueprint, abort

from ..schemas.user import UserSchema
from .methodviews import ProtectedMethodView

blueprint = Blueprint('profile', 'profile')


@blueprint.route('', endpoint='profile')
class ProfileAPI(ProtectedMethodView):
    @blueprint.response(UserSchema)
    def get(self):
        """Get user details"""
        User = current_app.user_datastore.user_model
        try:
            user = User.get(id=get_jwt_identity())
        except User.DoesNotExist:
            abort(404, message='User not found')
        return user

    @blueprint.arguments(UserSchema(partial=True))
    @blueprint.response(UserSchema)
    def patch(self, args):
        """Edit user details"""
        User = current_app.user_datastore.user_model
        try:
            user = User.get(id=get_jwt_identity())
        except User.DoesNotExist:
            abort(404, message='User not found')
        for field in args:
            setattr(user, field, args[field])
        if 'password' in args:
            user.password = hash_password(user.password)
        user.save()
        return user



import os
import socket
from importlib import import_module

from freenit import create_app

from config import configs
from name import app_name

api = import_module(f'{app_name}.api')
config_name = os.getenv('FLASK_ENV') or 'default'
config = configs[config_name]
application = create_app(config)
api.create_api(application)
hostname = socket.gethostname()
port = os.environ.get('FLASK_PORT', 5000)
SWAGGER_PATH = f'{config.OPENAPI_URL_PREFIX}{config.OPENAPI_SWAGGER_UI_PATH}'
SWAGGER_URL = f'http://{hostname}:{port}{SWAGGER_PATH}'

if __name__ == '__main__':
    print(' * Swagger URL:', SWAGGER_URL)
    application.run(
        host='0.0.0.0',
        port=port,
        debug=True,
        use_reloader=True,
    )



from flask import current_app
from flask_security.utils import hash_password
from flask_smorest import Blueprint, abort

from ..schemas.paging import PageInSchema, paginate
from ..schemas.user import UserPageOutSchema, UserSchema
from .methodviews import ProtectedMethodView

blueprint = Blueprint('users', 'user')


@blueprint.route('', endpoint='list')
class UserListAPI(ProtectedMethodView):
    @blueprint.arguments(PageInSchema(), location='headers')
    @blueprint.response(UserPageOutSchema)
    def get(self, pagination):
        """List users"""
        User = current_app.user_datastore.user_model
        if current_app.dbtype == 'sql':
            return paginate(User.select(), pagination)
        else:
            return paginate(User.objects.all(), pagination)

    @blueprint.arguments(UserSchema)
    @blueprint.response(UserSchema)
    def post(self, args):
        """Create user"""
        User = current_app.user_datastore.user_model
        user = User(**args)
        user.password = hash_password(user.password)
        user.save()
        return user


@blueprint.route('/<user_id>', endpoint='detail')
class UserAPI(ProtectedMethodView):
    @blueprint.response(UserSchema)
    def get(self, user_id):
        """Get user details"""
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                user = User.get(id=user_id)
            else:
                user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            abort(404, message='User not found')
        return user

    @blueprint.arguments(UserSchema(partial=True))
    @blueprint.response(UserSchema)
    def patch(self, args, user_id):
        """Edit user details"""
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                user = User.get(id=user_id)
            else:
                user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            abort(404, message='User not found')
        for field in args:
            setattr(user, field, args[field])
        if 'password' in args:
            user.password = hash_password(user.password)
        user.save()
        return user

    @blueprint.response(UserSchema)
    def delete(self, user_id):
        """Delete user"""
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                user = User.get(id=user_id)
            else:
                user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            abort(404, message='User not found')
        user.delete_instance()
        return user



from flask_security import UserMixin
from freenit.db import db
from peewee import BooleanField, DateTimeField, TextField

Model = db.Model


class User(Model, UserMixin):
    class Meta:
        table_name = 'users'

    active = BooleanField(default=False)
    admin = BooleanField(default=False)
    confirmed_at = DateTimeField(null=True)
    email = TextField(unique=True)
    password = TextField()



from marshmallow import fields

from .base import BaseSchema
from .role import UserRolesSchema


class TokenSchema(BaseSchema):
    email = fields.Email(required=True, description='Email')
    password = fields.Str(required=True, description='Password')


class BaseUserSchema(BaseSchema):
    active = fields.Boolean(description='Activate the user')
    admin = fields.Boolean(description='Is the user admin?')
    email = fields.Email(required=True, description='Email')
    password = fields.Str(
        required=True,
        description='Password',
        load_only=True
    )
    roles = fields.List(
        fields.Nested(UserRolesSchema),
        many=True,
        dump_only=True,
    )
    confirmed_at = fields.DateTime(
        description='Time when user was confirmed',
        dump_only=True,
    )


class RefreshSchema(BaseSchema):
    access = fields.Str()
    accessExpire = fields.Integer()
    refreshExpire = fields.Integer()


class LoginSchema(RefreshSchema):
    refresh = fields.Str()


class ResetSchema(BaseSchema):
    token = fields.String(required=True)
    password = fields.String(required=True)



import peewee as pw

SQL = pw.SQL


def migrate(migrator, database, fake=False, **kwargs):
    @migrator.create_model
    class Role(pw.Model):
        id = pw.AutoField()
        description = pw.TextField(null=True)
        name = pw.CharField(max_length=255, unique=True)

        class Meta:
            table_name = "role"

    @migrator.create_model
    class User(pw.Model):
        id = pw.AutoField()
        active = pw.BooleanField(default=False)
        admin = pw.BooleanField(default=False)
        confirmed_at = pw.DateTimeField(null=True)
        email = pw.TextField(unique=True)
        password = pw.TextField()
        reset = pw.TextField(null=True)

        class Meta:
            table_name = "users"

    @migrator.create_model
    class UserRoles(pw.Model):
        id = pw.AutoField()
        role = pw.ForeignKeyField(
            backref='users',
            column_name='role_id',
            field='id',
            model=migrator.orm['role']
        )
        user = pw.ForeignKeyField(
            backref='roles',
            column_name='user_id',
            field='id',
            model=migrator.orm['users']
        )

        class Meta:
            table_name = "userroles"


def rollback(migrator, database, fake=False, **kwargs):
    migrator.remove_model('userroles')
    migrator.remove_model('users')
    migrator.remove_model('role')



import json

import pytest
from flask import url_for
from werkzeug.datastructures import Headers


@pytest.mark.usefixtures('client_class')
class Base:
    headers = None

    def set_csrf(self, csrf_item=None):
        if csrf_item is not None:
            csrf_name = 'csrf_{}_token'.format(csrf_item)
            csrf = getattr(self, csrf_name)
            self.headers = Headers()
            self.headers.add_header('X-CSRF-TOKEN', csrf)

    def get(self, endpoint, **kwargs):
        url = url_for(endpoint, **kwargs)
        return self.client.get(url)

    def post(self, endpoint, data, csrf=None, **kwargs):
        url = url_for(endpoint, **kwargs)
        self.set_csrf(csrf)
        response = self.client.post(
            url,
            data=json.dumps(data),
            content_type='application/json',
            headers=self.headers,
        )
        self.headers = None
        return response

    def put(self, endpoint, data, csrf=None, **kwargs):
        url = url_for(endpoint, **kwargs)
        self.set_csrf(csrf)
        response = self.client.put(
            url,
            data=json.dumps(data),
            content_type='application/json',
            headers=self.headers,
        )
        self.headers = None
        return response

    def patch(self, endpoint, data, csrf=None, **kwargs):
        url = url_for(endpoint, **kwargs)
        self.set_csrf(csrf)
        response = self.client.patch(
            url,
            data=json.dumps(data),
            content_type='application/json',
            headers=self.headers,
        )
        self.headers = None
        return response

    def delete(self, endpoint, csrf=None, **kwargs):
        url = url_for(endpoint, **kwargs)
        self.set_csrf(csrf)
        return self.client.delete(url, headers=self.headers)

    def login(self, endpoint, user, password):
        data = {
            'email': user.email,
            'password': password,
        }
        response = self.post(endpoint, data)
        cookies = [
            header for header in response.headers if header[0] == 'Set-Cookie'
        ]
        for cookie in cookies:
            name, value = cookie[1].split(';')[0].split('=')
            setattr(self, name, value)
        return response



import click
from flask import current_app
from flask.cli import AppGroup
from flask_security.utils import hash_password

admin_group = AppGroup('admin', short_help='Manage admin users')
migration = AppGroup('migration', short_help='Migration operations')


def register_admin(app):
    @admin_group.command()
    def create():
        User = current_app.user_datastore.user_model
        try:
            if app.dbtype == 'sql':
                User.get(email='admin@example.com')
            else:
                User.objects.get(email='admin@example.com')
        except User.DoesNotExist:
            admin = User(
                email='admin@example.com',
                admin=True,
                active=True,
                password=hash_password('Sekrit'),
            )
            admin.save()

    app.cli.add_command(admin_group)


def register_migration(app):
    from peewee_migrate import Router
    project_root = app.config.get('PROJECT_ROOT', '')
    migrate_dir = f'{project_root}/migrations'
    router = Router(
        app.db.database,
        migrate_dir=f'{migrate_dir}/main',
    )

    @migration.command()
    def list():
        print('=== MAIN ===')
        for migration in router.done:
            print(migration)

    @migration.command()
    @click.argument('name')
    def create(name):
        app_name = app.config.get('NAME', '')
        router.create(name, f'{app_name}.models')

    @migration.command()
    def run():
        router.run()

    app.cli.add_command(migration)


def register(app):
    register_admin(app)
    if app.dbtype == 'sql':
        register_migration(app)



from smtplib import SMTP


def sendmail(config, message):
    port = config['MAIL'].get('port', 587)
    ssl = config['MAIL'].get('ssl', True)
    host = config['MAIL'].get('host', None)
    username = config['MAIL'].get('username', None)
    password = config['MAIL'].get('password', None)
    if None not in [host, username, password]:
        server = SMTP(host=host, port=port)
        if ssl:
            server.ehlo()
            server.starttls()
            server.ehlo()
        server.login(username, password)
        to = message.get('To', [])
        bcc = message.get('Bcc', [])
        if not isinstance(to, list):
            to = [to]
        if not isinstance(bcc, list):
            bcc = [bcc]
        to += bcc
        server.sendmail(
            message['From'],
            to,
            message.as_string().encode('utf-8')
        )



import os
from importlib import import_module

from freenit import create_app
from peewee_migrate import Router
from peewee_migrate.router import DEFAULT_MIGRATE_DIR

import pytest
from config import configs
from name import app_name
from pytest_factoryboy import register

from .factories import AdminFactory, RoleFactory, UserFactory

register(UserFactory)
register(AdminFactory)
register(RoleFactory)

schemas = {
    'user': f'{app_name}.schemas.user',
}


@pytest.fixture
def app():
    api = import_module(f'{app_name}.api')
    config = configs['testing']
    flask_app = create_app(config, schemas=schemas)
    api.create_api(flask_app)
    router = Router(
        flask_app.db.database,
        migrate_dir=f'{DEFAULT_MIGRATE_DIR}/main',
    )
    #  log_router = Router(
    #      flask_app.logdb.database,
    #      migrate_dir=f'{DEFAULT_MIGRATE_DIR}/logs',
    #  )
    router.run()
    #  log_router.run()
    yield flask_app
    flask_app.db.close_db('')
    #  flask_app.logdb.close_db('')
    current_path = os.path.dirname(__file__)
    os.remove('{}/../test.db'.format(current_path))
    #  os.remove('{}/../log_test.db'.format(current_path))



import functools

from flask import current_app
from flask_jwt_extended import get_jwt_identity
from flask_smorest import abort


def get_user():
    User = current_app.user_datastore.user_model
    user_id = get_jwt_identity()
    if user_id is None:
        abort(401, message='No token provided')
    try:
        if current_app.dbtype == 'sql':
            user = User.get(id=user_id)
        else:
            user = User.objects.get(id=user_id)
    except User.DoesNotExist:
        abort(404, message='No such user')
    return user


def every(roles):
    def outer(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            user = get_user()
            if not user.admin:
                if current_app.dbtype == 'sql':
                    user_roles = [role.role.name for role in user.roles]
                else:
                    user_roles = [role.name for role in user.roles]
                for role in roles:
                    if role not in user_roles:
                        abort(403, message='Access denied')
            return func(*args, **kwargs)

        return inner

    return outer


def one(roles):
    def outer(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            user = get_user()
            if user.admin:
                return func(*args, **kwargs)
            if current_app.dbtype == 'sql':
                user_roles = [role.role.name for role in user.roles]
            else:
                user_roles = [role.name for role in user.roles]
            for role in roles:
                if role in user_roles:
                    return func(*args, **kwargs)
            abort(403, message='Access denied')

        return inner

    return outer



from .base import Base


class TestRole(Base):
    def test_get_all(self, user_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.get('roles.list')
        assert response.status_code == 200

    def test_post(self, user_factory, role_factory):
        user = user_factory()
        role = role_factory()
        role.save()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200

        data = {
            'description': 'Admin role.',
            'name': 'Admin',
        }

        response = self.post('roles.list', data, 'access')
        assert response.status_code == 200
        json_data = response.get_json()
        assert data['name'] == json_data['name']
        newrole = role.get(id=json_data['id'])
        assert newrole.description == data['description']

    def test_get(self, user_factory, role_factory):
        user = user_factory()
        role = role_factory()
        user.save()
        role.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        response = self.get('roles.detail', role_id=role.id)
        assert response.status_code == 200

    def test_patch(self, user_factory, role_factory):
        user = user_factory()
        role = role_factory()
        user.save()
        role.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        data = {
            'description': 'Admin role',
        }
        response = self.patch('roles.detail', data, 'access', role_id=role.id)
        assert response.status_code == 200
        role = role.get(id=role.id)
        assert role.description == data['description']

    def test_delete(self, user_factory, role_factory):
        user = user_factory()
        user.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        newrole = role_factory()
        newrole.save()
        response = self.delete('roles.detail', 'access', role_id=newrole.id)
        assert response.status_code == 200

    def test_assign_deassign_role(self, user_factory, role_factory):
        user = user_factory()
        role = role_factory()
        user.save()
        role.save()
        response = self.login('auth.login', user, 'Sekrit')
        assert response.status_code == 200
        user = user_factory()
        user.save()
        data = {
            "id": user.id
        }
        response = self.post('roles.user_assign',
                             data, 'access',
                             role_id=role.id)
        assert response.status_code == 200
        response = self.delete('roles.user_deassign',
                               'access',
                               user_id=user.id,
                               role_id=role.id)
        assert response.status_code == 200



import os

from name import app_name

SECRET_KEY = 'iQfPvB6sZaNHqVFI5CJa9rM1xOEVHKIM0LwifT04yLsPlZhSSvaDuZXOgJFSpJVq'


class Config:
    NAME = app_name
    PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
    DEBUG = False
    SECURITY_PASSWORD_SALT = 'freenit'
    SECRET_KEY = SECRET_KEY
    SECURITY_TRACKABLE = False
    JWT_SECRET_KEY = SECRET_KEY
    JWT_TOKEN_LOCATION = ['cookies']
    JWT_ACCESS_COOKIE_PATH = '/api/v0'
    JWT_REFRESH_COOKIE_PATH = '/api/v0/auth/refresh'
    JWT_SESSION_COOKIE = False
    JWT_COOKIE_SECURE = True
    OPENAPI_URL_PREFIX = '/doc'
    OPENAPI_REDOC_PATH = '/redoc'
    OPENAPI_SWAGGER_UI_PATH = '/swaggerui'
    OPENAPI_SWAGGER_UI_URL = '/static/swaggerui/'
    OPENAPI_VERSION = '3.0.3'
    MEDIA_URL = '/media'
    MEDIA_PATH = 'media'
    ACCOUNT_REQUEST_EXPIRY = 24  # in hours
    PASSWORD_RESET_EXPIRY = 2  # in hours
    DATABASE = {
        'name': 'database.db',
        'engine': 'SqliteDatabase',
    }
    MONGODB_SETTINGS = {
        'host': 'mongodb',
        'db': 'freenit',
    }
    MAIL = {
        #  'host': 'mail.example.com',
        #  'port': 587,
        #  'ssl': True,
        #  'username': 'someone@example.com',
        #  'password': 'Sekrit',
    }
    FROM_EMAIL = 'office@example.com'
    SUBJECTS = {
        'prefix': '[Freenit] ',
        'confirm': 'Account confirmation',
        'register': 'Account registration',
    }

    @staticmethod
    def init_app(app):
        pass


class DevConfig(Config):
    DEBUG = True
    JWT_COOKIE_SECURE = False
    SECURITY_SEND_REGISTER_EMAIL = False


class TestConfig(Config):
    TESTING = True
    JWT_COOKIE_SECURE = False
    DATABASE = {
        'name': 'test.db',
        'engine': 'SqliteDatabase',
    }


class ProdConfig(Config):
    pass



from apispec.ext.marshmallow import MarshmallowPlugin
from apispec.ext.marshmallow.common import resolve_schema_cls
from flask import render_template
from flask_smorest import Api


class MyApi(Api):
    def _openapi_swagger_ui(self):
        return render_template('swaggerui.html', title=self._app.name)


def register_endpoints(app, api_prefix, blueprints):
    for blueprint in blueprints:
        app.api.register_blueprint(
            blueprint,
            url_prefix='{}/{}'.format(
                api_prefix,
                blueprint.name,
            ),
        )


def schema_name_resolver(schema):
    schema_cls = resolve_schema_cls(schema)
    name = schema_cls.__name__
    if name.endswith("Schema"):
        name = name[:-6] or name
    if getattr(schema, 'partial', None):
        if isinstance(schema.partial, list):
            for field in schema.partial:
                name += field.capitalize()
        name += 'Partial'
    if getattr(schema, 'only', None):
        for field in schema.only:
            name += field.capitalize()
        name += 'Only'
    if getattr(schema, 'exclude', None):
        for field in schema.exclude:
            name += field.capitalize()
        name += 'Exclude'
    return name


def create_api(app):
    from .auth import blueprint as auth
    from .profile import blueprint as profile
    from .role import blueprint as role
    from .user import blueprint as user

    marshmallow_plugin = MarshmallowPlugin(
        schema_name_resolver=schema_name_resolver
    )
    spec_kwargs = {
        'marshmallow_plugin': marshmallow_plugin,
    }
    app.api = MyApi(app, spec_kwargs=spec_kwargs)
    register_endpoints(
        app,
        '/api/v0',
        [
            auth,
            profile,
            role,
            user,
        ],
    )



#!/usr/bin/env python3

import pathlib

from setuptools import find_packages, setup

PROJECT_ROOT = pathlib.Path(__file__).parent
README = (PROJECT_ROOT / 'README.md').read_text()

ldap = ['ldap3']
mongo = ['flask-mongoengine']
sql = [
    'peewee',
    'peewee-migrate>=1.1.6',
]

setup(
    name='freenit',
    version='0.1.9',
    description='REST API framework based on Flask-Smorest',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/freenit-framework/backend',
    author='Goran Mekić',
    author_email='meka@tilda.center',
    license='BSD',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords=[
        'REST',
        'openapi',
        'swagger',
        'flask',
        'marshmallow',
        'apispec'
        'webargs',
    ],
    packages=find_packages(exclude=['tests*']),
    python_requires='>=3.5',
    install_requires=[
        'bcrypt',
        'flask-collect>=1.3.2',
        'flask-cors>=2.1.2',
        'flask-jwt-extended>=3.24.1',
        'flask-security>=3.0.0',
        'flask-smorest>=0.18.2',
        'email-validator',
    ],
    extras_require={
        'all': ldap + mongo + sql,
        'sql': sql,
        'mongo': mongo,
    },
    include_package_data=True,
    package_data={
        '': [
            'project/*',
            'project/ansible/group_vars/*',
            'project/ansible/inventory/*',
            'project/ansible/roles/devel/*',
            'project/bin/*',
            'project/project/*',
            'project/project/api/*',
            'project/project/models/*',
            'project/project/schemas/*',
            'project/project/templates/*',
            'project/tests/*',
            'static/swaggerui/*',
            'templates/*',
        ]
    },
    scripts=['bin/freenit.sh'],
)



from marshmallow import fields

from ..fields.objectid import ID
from .base import BaseSchema


class RoleBaseSchema(BaseSchema):
    description = fields.String(description='Description')
    name = fields.String(description='Name')


class BaseRoleSchema(RoleBaseSchema):
    users = fields.List(fields.Nested('UserSchema'), dump_only=True)


class UserRolesSchema(BaseSchema):
    role = fields.Nested('RoleSchema')


class UserAssignSchema(BaseSchema):
    id = ID(description='ID')



from datetime import datetime, timedelta
from email.mime.text import MIMEText

from flask import current_app, jsonify, request
from flask.views import MethodView
from flask_jwt_extended import (
    create_access_token,
    create_refresh_token,
    decode_token,
    get_jwt_identity,
    jwt_refresh_token_required,
    set_access_cookies,
    set_refresh_cookies,
    unset_jwt_cookies
)
from flask_security.utils import hash_password, verify_password
from flask_smorest import Blueprint, abort

from ..schemas.user import (
    LoginSchema,
    RefreshSchema,
    ResetSchema,
    TokenSchema,
    UserSchema
)

blueprint = Blueprint('auth', 'auth')


@blueprint.route('/login', endpoint='login')
class AuthLoginAPI(MethodView):
    @blueprint.response(LoginSchema)
    @blueprint.arguments(TokenSchema)
    def post(self, args):
        """Authenticates and generates a token"""
        email = args.get('email', None)
        password = args.get('password', None)
        if email is None:
            abort(403, message='Email not provided')
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                user = User.get(email=email)
            else:
                user = User.objects.get(email=email)
        except User.DoesNotExist:
            abort(403, message='No such user, or wrong password')
        if not user or not user.active:
            abort(403, message='No such user, or wrong password')
        if not verify_password(password, user.password):
            abort(403, message='No such user, or wrong password')
        if current_app.dbtype == 'sql':
            id = user.id
        else:
            id = str(user.id)
        access_token = create_access_token(identity=id)
        refresh_token = create_refresh_token(identity=id)
        access_expire = current_app.config['JWT_ACCESS_TOKEN_EXPIRES']
        refresh_expire = current_app.config['JWT_REFRESH_TOKEN_EXPIRES']
        resp = jsonify(
            {
                'access': access_token,
                'refresh': refresh_token,
                'accessExpire': int(access_expire.total_seconds()),
                'refreshExpire': int(refresh_expire.total_seconds()),
            }
        )
        set_access_cookies(resp, access_token)
        set_refresh_cookies(resp, refresh_token)
        refresh_path = current_app.config['JWT_REFRESH_COOKIE_PATH']
        refresh_secure = current_app.config['JWT_COOKIE_SECURE']
        refresh_expire_date = datetime.now() + refresh_expire
        resp.set_cookie(
            'refresh_expire',
            value=str(refresh_expire_date),
            expires=refresh_expire_date,
            path=refresh_path,
            httponly=True,
            secure=refresh_secure,
        )
        return resp


@blueprint.route('/logout', endpoint='logout')
class AuthLogoutAPI(MethodView):
    def post(self):
        """Logout"""
        resp = jsonify({})
        unset_jwt_cookies(resp)
        return resp


@blueprint.route('/refresh', endpoint='refresh')
class AuthRefreshAPI(MethodView):
    @blueprint.response(RefreshSchema)
    @jwt_refresh_token_required
    def post(self):
        """Refresh access token"""
        identity = get_jwt_identity()
        User = current_app.user_datastore.user_model
        try:
            if current_app.dbtype == 'sql':
                user = User.get(id=identity)
            else:
                user = User.objects.get(id=identity)
        except User.DoesNotExist:
            abort(403, message='No such user, or wrong password')
        if not user.active:
            abort(403, message='No such user, or wrong password')
        access_expire = current_app.config['JWT_ACCESS_TOKEN_EXPIRES']
        access_token = create_access_token(identity=identity)
        refresh_expire_date = datetime.strptime(
            request.cookies['refresh_expire'],
            '%Y-%m-%d %H:%M:%S.%f'
        )
        refresh_delta = refresh_expire_date - datetime.now()
        resp = jsonify(
            {
                'access': access_token,
                'accessExpire': int(access_expire.total_seconds()),
                'refreshExpire': int(refresh_delta.total_seconds()),
            }
        )
        set_access_cookies(resp, access_token)
        return resp


@blueprint.route('/register', endpoint='register')
class AuthRegisterAPI(MethodView):
    @blueprint.response(UserSchema)
    @blueprint.arguments(TokenSchema)
    def post(self, args):
        """Register new user"""
        email = args['email']
        password = args['password']
        User = current_app.user_datastore.user_model
        user = None
        try:
            User.get(email=email)
            abort(409, message='User already registered')
        except User.DoesNotExist:
            user = User(email=email, password=hash_password(password))
            user.save()
        expires = timedelta(
            hours=current_app.config['ACCOUNT_REQUEST_EXPIRY'],
        )
        identity = {
            'id': user.id,
            'request': True,
        }
        host = request.headers.get('Origin', request.url_root)
        requestToken = create_access_token(identity, expires_delta=expires)
        confirm = 'confirm'
        if host[-1] != '/':
            confirm = f'/{confirm}'
        url = f'{host}{confirm}/{requestToken}'
        msg = MIMEText(url, 'plain', 'utf-8')
        config = current_app.config
        subject = config['SUBJECTS']['prefix'] + config['SUBJECTS']['register']
        msg['To'] = user.email
        msg['From'] = config['FROM_EMAIL']
        msg['Subject'] = subject
        current_app.sendmail(msg)
        return user


@blueprint.route('/register/<token>', endpoint='register')
class AuthRegisterConfirmAPI(MethodView):
    @blueprint.response(UserSchema)
    def get(self, token):
        """Confirm new user"""
        decoded_token = decode_token(token)
        identity = decoded_token['identity']
        User = current_app.user_datastore.user_model
        user = None
        try:
            user = User.get(id=identity['id'])
        except User.DoesNotExist:
            abort(404, message='User does not exist')
        if user.active:
            abort(409, message='User already activated')
        user.active = True
        user.save()
        text = 'Congratulation, your account is confirmed'
        msg = MIMEText(text, 'plain', 'utf-8')
        config = current_app.config
        subject = config['SUBJECTS']['prefix'] + config['SUBJECTS']['confirm']
        msg['To'] = user.email
        msg['From'] = config['FROM_EMAIL']
        msg['Subject'] = subject
        current_app.sendmail(msg)
        return user


@blueprint.route('/reset/request', endpoint='reset_request')
class AuthResetRequestAPI(MethodView):
    @blueprint.response(TokenSchema)
    @blueprint.arguments(TokenSchema(only=('email', )))
    def post(self, args):
        """Request user password reset"""
        User = current_app.user_datastore.user_model
        try:
            user = User.get(email=args['email'], active=True)
            expires = timedelta(
                hours=current_app.config['PASSWORD_RESET_EXPIRY'],
            )
            identity = {
                'id': user.id,
                'reset': True,
            }
            host = request.headers.get('Origin', request.url_root)
            resetToken = create_access_token(identity, expires_delta=expires)
            url = f'{host}/reset/{resetToken}'
            msg = MIMEText(url, 'plain', 'utf-8')
            msg['From'] = 'office@example.com'
            subjects = current_app.config['SUBJECTS']
            subject = subjects['prefix'] + subjects['register']
            msg['Subject'] = subject
            msg['To'] = user.email
            current_app.sendmail(msg)
        except User.DoesNotExist:
            pass
        return {}


@blueprint.route('/reset', endpoint='reset')
class AuthResetAPI(MethodView):
    @blueprint.response(ResetSchema)
    @blueprint.arguments(ResetSchema)
    def post(self, args):
        """Reset user password"""
        decoded_token = decode_token(args['token'])
        identity = decoded_token['identity']
        if not identity.get('reset', False):
            abort(409, message='Not reset token')
        User = current_app.user_datastore.user_model
        try:
            user = User.get(id=identity['id'], active=True)
        except User.DoesNotExist:
            abort(404, message='No such user')
        user.password = hash_password(args['password'])
        user.save()
        return {}



from importlib import import_module

from flask_security.utils import hash_password

import factory
from name import app_name

user = import_module(f'{app_name}.models.user')
role = import_module(f'{app_name}.models.role')


class UserFactory(factory.Factory):
    class Meta:
        model = user.User

    first_name = factory.Faker('first_name')
    last_name = factory.Faker('last_name')
    email = factory.Faker('email')
    password = factory.LazyAttribute(lambda a: hash_password('Sekrit'))
    username = factory.Faker('name')
    active = True


class AdminFactory(UserFactory):
    admin = True


class RoleFactory(factory.Factory):
    class Meta:
        model = role.Role

    name = factory.Faker('first_name')



from marshmallow import ValidationError, fields


class ID(fields.Field):
    def _serialize(self, value, attr, obj, *args, **kwargs):
        if type(value) == int:
            return value
        if value is None:
            return None
        return str(value)

    def _deserialize(self, value, attr, data, *args, **kwargs):
        if type(value) == int:
            return value
        try:
            import bson
            return bson.ObjectId(value)
        except (TypeError, bson.errors.InvalidId):
            raise ValidationError('Invalid ObjectId.')
